@@ -21,12 +21,15 @@ from os.path import join, abspath, dirname, splitext, split, exists
 import inspect
 
 import cherrypy
+from cherrypy.lib.static import serve_file
 from cherrypy.process.plugins import PIDFile
 
 from ion.controllers import Controller
 from sqlalchemy_tool import metadata, session, mapper, configure_session_for_app
 from ion.context import Context
 from ion.cache import Cache
+from ion.fs import imp as import_module
+from ion.fs import locate, is_file
 from sqlalchemy.exc import DBAPIError
 import logging
 
@@ -38,7 +41,11 @@ class ServerStatus(object):
     Stopped = 4
 
 class Server(object):
-    imp = __import__
+    import_method = __import__
+
+    @classmethod
+    def imp(cls, name):
+        return import_module(name, Server.import_method)
 
     def __init__(self, root_dir, context=None):
         self.status = ServerStatus.Unknown
@@ -48,12 +55,6 @@ class Server(object):
         self.test_connection_error = None
         self.cache = None
 
-    @property
-    def template_path(self):
-        templ_path = self.context.settings.Ion.template_path.lstrip("/")
-        templ_path = templ_path and abspath(join(self.root_dir, templ_path)) or abspath(join(self.root_dir, 'templates'))
-        return templ_path
-
     def start(self, config_path, non_block=False):
         self.status = ServerStatus.Starting
         self.publish('on_before_server_start', {'server':self, 'context':self.context})
@@ -61,10 +62,17 @@ class Server(object):
         self.context.load_settings(abspath(join(self.root_dir, config_path)))
         self.cache = Cache(size=1000, age="5s", log=cherrypy.log)
 
+        self.context.load_apps()
+        
+        self.apps = self.context.apps
+        self.app_paths = self.context.app_paths
+        self.app_modules = self.context.app_modules
+        
         if self.context.settings.Ion.as_bool('debug'):
             logging.basicConfig()
             logging.getLogger('sqlalchemy.engine').setLevel(logging.INFO)
-            logging.getLogger('sqlalchemy.orm.unitofwork').setLevel(logging.DEBUG)
+            logging.getLogger('sqlalchemy.orm.unitofwork').\
+                                                    setLevel(logging.DEBUG)
 
         self.import_template_filters()
         self.import_controllers()
@@ -79,28 +87,30 @@ class Server(object):
         self.publish('on_after_server_start', {'server':self, 'context':self.context})
 
     def import_template_filters(self):
-        if exists(abspath(join(self.root_dir, "template_filters.py"))):
-            template_filters = Server.imp("template_filters")
-            for name, func in inspect.getmembers(template_filters):
-                if inspect.isfunction(func) and not name.startswith("_"):
-                    self.template_filters[name] = func
+
+        for app in self.apps:
+            template_filters_module = Server.imp("%s.template_filters" %app)
+            if template_filters_module:
+                for name, func in inspect.getmembers(template_filters_module):
+                    if inspect.isfunction(func) and not name.startswith("_"):
+                        self.template_filters[name] = func
 
     def import_controllers(self):
-        controller_path = self.context.settings.Ion.controllers_path
-        controller_path = controller_path.lstrip("/") or "controllers"
-        controller_path = abspath(join(self.root_dir, controller_path))
-
-        sys.path.append(controller_path)
-
-        for filename in os.listdir(controller_path):
-            if filename.endswith(".py"):
-                Server.imp(splitext(filename)[0])
+        for app in self.apps:
+            ctrl_module = Server.imp(app + '.controllers')
+            if not ctrl_module:
+                raise RuntimeError('The app %s does not have a controllers module.' % app)
 
     def stop(self):
         self.status = ServerStatus.Stopping
         self.publish('on_before_server_stop', {'server':self, 'context':self.context})
 
         cherrypy.engine.exit()
+        cherrypy.server.httpserver = None
+
+        self.apps = None
+        self.app_paths = None
+        self.app_modules = None
 
         self.status = ServerStatus.Stopped
         self.publish('on_after_server_stop', {'server':self, 'context':self.context})
@@ -123,16 +133,6 @@ class Server(object):
 
     def get_mounts(self, dispatcher):
         sets = self.context.settings
-        media_path = sets.Ion.media_path
-
-        if not media_path:
-            media_dir = "media"
-            media_path = self.root_dir
-        else:
-            #REFACTOR
-            paths = split(media_path)
-            media_dir = paths[-1]
-            media_path = join(self.root_dir, join(*paths[:-1]).lstrip("/")).rstrip("/")
 
         protocol = self.context.settings.Db.protocol
         username = self.context.settings.Db.user
@@ -146,15 +146,10 @@ class Server(object):
         conf = {
             '/': {
                 'request.dispatch': dispatcher,
-                'tools.staticdir.root': media_path,
                 'tools.SATransaction.on': True,
                 'tools.SATransaction.dburi':conn_str, 
                 'tools.SATransaction.echo': sets.Ion.as_bool('verbose'),
                 'tools.SATransaction.convert_unicode': True
-            },
-            '/media': {
-                'tools.staticdir.on': True,
-                'tools.staticdir.dir': media_dir
             }
         }
 
@@ -163,11 +158,45 @@ class Server(object):
     def get_dispatcher(self):
         routes_dispatcher = cherrypy.dispatch.RoutesDispatcher()
 
+        class MediaController():
+            def __init__(self, apps):
+                self.apps = apps
+
+            def serve_media(self, media_url):
+                for app in self.apps:
+                    extension = splitext(media_url)[-1]
+                    app_module = reduce(getattr, app.split('.')[1:], __import__('testapp'))
+                    path = inspect.getfile(app_module)
+                    media_path = abspath(join("/".join(split(path)[:-1]), 'media', media_url))
+
+                    if exists(media_path):
+                        if extension == ".jpg":
+                            content_type = "image/jpeg"
+                        elif extension == ".gif":
+                            content_type = "image/gif"
+                        elif extension == ".png":
+                            content_type = "image/png"
+                        elif extension == ".js":
+                            content_type = "text/javascript"
+                        elif extension == ".css":
+                            content_type = "text/css"
+                        elif extension.startswith('.htm'):
+                            content_type = "text/html"
+                        else:
+                            content_type = "text/plain"
+
+                        return serve_file(media_path, content_type=content_type)
+
+                raise cherrypy.HTTPError(404)
+
         for controller_type in Controller.all():
             controller = controller_type()
             controller.server = self
             controller.register_routes(routes_dispatcher)
 
+        media_controller = MediaController(self.apps)
+        routes_dispatcher.connect("media", "/media/{media_url:(.+)}", controller=media_controller, action="serve_media")
+
         route_name = "healthcheck"
         controller = Controller()
         controller.server = self
