@@ -1,6 +1,7 @@
 #!/usr/bin/env python
 #-*- coding:utf-8 -*-
 
+import sys
 from os.path import abspath, join, split, dirname
 
 from ion import Server, ServerStatus, Context
@@ -15,6 +16,10 @@ class ServerHelper (object):
         while not self.server.status == ServerStatus.Started:
             time.sleep(0.5)
 
+    def stop(self):
+        self.server.stop()
+        self.server = None
+
     def ctrl(self, controller):
         _ctrl = controller()
         _ctrl.server = self.server
